@@ -8,32 +8,32 @@
 #' @param pred.data a \code{data.frame} that holds prediction points, must have
 #'        the correct columns for other environmental covariates. It also MUST
 #'        have a column called \code{cell.area} which gives the area for each
-#'        prediction cell 
+#'        prediction cell
 #' @param n.boot number of bootstrap resamples.
 #' @param block.size number of segments in each block.
-#' @param off.set offset to be used in the model, see \code{\link{offsets}}.
+#' @param off.set offset to be used in the model, see \code{\link{offsets}}
 #'        Note that this should NOT be \code{log()}'d.
 #' @param ds.uncertainty incorporate uncertainty in the detection function? See
 #'        Details, below. Note that this feature is EXPERIMENTAL at the moment.
-#' @param samp.unit.name name sampling unit to resample (default 
+#' @param samp.unit.name name sampling unit to resample (default
 #'        'Transect.Label').
-#' @param progress.file path to a file to be used (usually by Distance) to 
+#' @param progress.file path to a file to be used (usually by Distance) to
 #'        generate a progress bar (default \code{NULL} -- no file written).
 #' @param bs.file path to a file to store each boostrap round. This stores all
 #'        of the bootstrap results rather than just the summaries, enabling
 #'        outliers to be detected and removed. (Default \code{NULL}).
 #' @param bar should a progress bar be printed to screen? (Default \code{TRUE}).
 #'
-#' @section Details: 
-#'  Setting \code{ds.uncertainty=TRUE} will incorporate detection function 
-#'  uncertainty directly into the bootstrap. This is done by generating 
-#'  observations from the fitted detection function and then re-fitting a new 
-#'  detection function (of the same form), then calculating a new effective 
+#' @section Details:
+#'  Setting \code{ds.uncertainty=TRUE} will incorporate detection function
+#'  uncertainty directly into the bootstrap. This is done by generating
+#'  observations from the fitted detection function and then re-fitting a new
+#'  detection function (of the same form), then calculating a new effective
 #'  strip width. Rejection sampling is used to generate the observations (except
 #'  in the half-normal case) so the procedure can be rather slow. Note that this
 #'  is currently not supported with covariates in the detection function.
 #'
-#'  Setting \code{ds.uncertainty=FALSE} will incorporate detection function 
+#'  Setting \code{ds.uncertainty=FALSE} will incorporate detection function
 #'  uncertainty using the delta method. This assumes that the detection function
 #'  and the spatial model are INDEPENDENT. This is probably not reasonable.
 #'
@@ -83,7 +83,7 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
   # function uncertainty
   if(ds.uncertainty &
      dsm.object$ddf$ds$aux$ddfobj$scale$formula != "~1"){
-    stop("Detection function uncertainty with covariates is not supported")
+    stop("Cannot incorporate detection function uncertainty with covariates in the detection function")
   }
 
   # Initialize storage
@@ -93,22 +93,28 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
 
   # save the original off.set that was supplied
   original.offset<-off.set
+  # extract the link and inverse link functions
+  linkfn <- dsm.object$family$linkfun
+  invlinkfn <- dsm.object$family$linkinv
+  # what was the response?
+  response <- as.character(dsm.object$formula)[2]
 
   # Sort out sampling unit for dsm object
   dsm.object$data$sampling.unit <- dsm.object$data[[samp.unit.name]]
 
-  # Following line removes any transect into which missing data was 
+  # Following line removes any transect into which missing data was
   # detected by the call to gam and recorded in '$na.action'
   # Consequence of this step should be that no sampling unit (transect)
   # can become part of the bootstrap when any segment has missing data.
   name.sampling.unit <- unique(dsm.object$data$sampling.unit)
   num.sampling.unit <- length(name.sampling.unit)
 
-  # Get residuals 
-  obs <- dsm.object$data[[as.character(dsm.object$formula)[2]]]
+  # Get residuals
+  obs <- dsm.object$data[[response]]
   fit.vals <- rep(NA,length(obs))
   fit.vals[!is.na(obs)] <- fitted(dsm.object)
-  dsm.object$data$log.resids <- dsm.object$family$linkfun(obs)
+  eps <- 0.001
+  dsm.object$data$log.resids <- linkfn(obs+eps)-linkfn(fit.vals+eps)
 
   # Sort out blocks for each sampling unit
   block.info <- block.info.per.su(block.size=block.size,
@@ -147,19 +153,20 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
     }
   }
 
+  ## prediction from the observed data
   dsm.predict.bs <- try(predict(dsm.object,
                                 newdata=pred.data,
                                 off.set=off.set))
+  # we shouldn't get errors in this predict, but check just in case
   if(all(class(dsm.predict.bs)=="try-error")){
     dsm.predict.bs <- rep(NA,length(short.var$sumx))
   }
 
-  # Don't save all cell values for all reps, rather, 
+  # Don't save all cell values for all reps, rather,
   #  populate dataframe with machine formula components for each cell
-  vector.cell.abundances <- dsm.predict.bs
-  short.var$sumx <- short.var$sumx + vector.cell.abundances
-  short.var$sumx.sq <- short.var$sumx.sq + vector.cell.abundances^2
-  study.area.total[1] <- sum(vector.cell.abundances, na.rm=TRUE)
+  short.var$sumx <- short.var$sumx + dsm.predict.bs
+  short.var$sumx.sq <- short.var$sumx.sq + dsm.predict.bs^2
+  study.area.total[1] <- sum(dsm.predict.bs, na.rm=TRUE)
 
   if(!is.null(bs.file)){
     # append to the file bs.file
@@ -167,28 +174,29 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
                 sep=",",col.names=FALSE)
   }
 
-  # save the offset
-  off.set.save <- off.set
+  # save the data used to fit the model
+  data.save <- dsm.object$data
 
   # Start bootstrapping
   for(i in 2:n.boot){
-    # Compute proportion of bootstrapping completed, write it to file 
-    # passed as argument 'progress.file'
+    # Compute proportion of bootstrapping completed, write it to file
+    # passed as argument 'progress.file'
     # This file will be read by Distance to present a progress bar.
     if(!is.null(progress.file)){
       progress <- round(i/n.boot, 2)* 100
       write(progress, file=progress.file, append=FALSE)
     }
 
+    # make the boostrap sample of the blocks
     bs.blocks <- sample(block.vector, num.blocks.required, replace=TRUE)
+    # extract the sample residuals from the data
     bs.resids <- generate.mb.sample(num.blocks.required, block.size,
                                     bs.blocks, dsm.object$data,
                                     block.info, num.sampling.unit)
+    # copy the data into a new object
+    bs.samp <- data.save
 
-    # Back transform to get bootstrap observations
-    bs.samp <- dsm.object$data
-
-    # if we are incirporating detection function uncertainty, then
+    # if we are incorporating detection function uncertainty, then
     # need to resample the distances
     if(ds.uncertainty){
 
@@ -205,46 +213,47 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
       new.p <- generate.ds.uncertainty(dsm.object$ddf)
 
       # calculate the new offset
-      this.offset <- new.p*(exp(dsm.object$data$off.set)/old.p)
+      this.offset <- new.p*(invlinkfn(dsm.object$data$off.set)/old.p)
 
       # calculate the fitted values with the new offset
-      fit <- (fitted(dsm.object)/exp(dsm.object$data$off.set))*
+      fit <- (fitted(dsm.object)/invlinkfn(dsm.object$data$off.set))*
                                     this.offset
 
-      # replace the offset in the model 
-      bs.samp$off.set <- log(this.offset)
-
-      # replace the offset in the prediction grid
-      off.set <- new.p*(off.set.save/old.p)
+      # replace the offset in the model
+      bs.samp$off.set <- linkfn(this.offset)
     }else{
-    # if we're not doing detection function uncertainty
+      # if we're not doing detection function uncertainty then
+      #  just get the fitted values
       fit <- fitted(dsm.object)
     }
 
-    if(sum(is.na(fit)) > 0 ){
-      stop(paste("Missing values detected in survey covariates,",
-                 " cannot be used with moving block"))
+    if(any(is.na(fit))){
+      stop("Data with NA covariate values can't be used with moving block")
     }
 
-    # calculate the new fitted values
-    bs.samp[[as.character(dsm.object$formula)[2]]] <- fit*exp(bs.resids)
+    # calculate the bootstrap "observations"
+    bs.samp[[response]] <- fit*invlinkfn(bs.resids)
+
 
     ## Fit model to dsm bootstrap sample
-
-    # Reconstruct dsm model fitting command -- this is a call to gam()
+    # Reconstruct dsm model fitting command -- this is a call to gam() or gamm()
     gam.call <- dsm.object$call
     gam.call$formula <- dsm.object$formula
     gam.call$family <- dsm.object$family
-
     ## put the bootstrap data into the gam call
-    gam.call$data<-bs.samp
-
-    # Handle chaos in gam fitting caused by pathological bootstrap resample
+    gam.call$data <- bs.samp
+    # fit model to bootstrap data
+    #   try() to handle model failure,
+    #   withCallingHandlers() to deal with possible spurious error message
+    #   with() is not required for gam but IS necessary for gamm models
+    #     to obtain covariance structure etc
+    #dsm.bs.model <- try(with(dsm.object,eval(gam.call)))
     dsm.bs.model <- try(with(dsm.object,withCallingHandlers(eval(gam.call),
                                            warning = matrixnotposdef.handler)))
 
+    # if everything didn't fail...
     if(all(class(dsm.bs.model)!="try-error")){
-
+      # make the model a dsm
       class(dsm.bs.model) <- c("dsm",class(dsm.bs.model))
 
       # Do prediction using newly fitted dsm model created from bootstrap sample
@@ -256,14 +265,14 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
       }
 
       # Don't save all cell values for all reps, rather,
-      #  populate dataframe with machine formula components for each cell
-      vector.cell.abundances <- dsm.predict.bs
-      short.var$sumx <- short.var$sumx + vector.cell.abundances
-      short.var$sumx.sq <- short.var$sumx.sq + vector.cell.abundances^2
-      study.area.total[i] <- sum(vector.cell.abundances, na.rm=TRUE)
+      #  populate data.frame with formula components for each cell
+      dsm.predict.bs <- dsm.predict.bs
+      short.var$sumx <- short.var$sumx + dsm.predict.bs
+      short.var$sumx.sq <- short.var$sumx.sq + dsm.predict.bs^2
+      study.area.total[i] <- sum(dsm.predict.bs, na.rm=TRUE)
 
       if(!is.null(bs.file)){
-        # append to the file bs.file
+        # append predictions to the file bs.file
         write.table(t(dsm.predict.bs),bs.file,append=TRUE,
                     sep=",",col.names=FALSE)
       }
@@ -271,6 +280,11 @@ dsm.var.movblk <- function(dsm.object, pred.data, n.boot, block.size,
       study.area.total[i] <- NA
     }
 
+    # the model and data might be quite big, so clear some space
+    rm(dsm.predict.bs,gam.call,bs.samp)
+    gc()
+
+    # add a tick to the progress bar
     if(bar){
       setTxtProgressBar(pb, i)
     }
