@@ -7,11 +7,13 @@ make.data <- function(response, ddfobject, segdata, obsdata, group,
   distance.name<-'distance'
   cluster.name<-'size'
 
-  # Truncate observations made at distances greater than the truncation width;
-  # truncation value picked up from metadata contained within ddfobject
-  # No truncation for strip transects
-  if (!is.null(ddfobject)){
-     obsdata <- obsdata[obsdata[,distance.name]<=ddfobject$meta.data$width,]
+  # Check that observations are between left and right truncation
+  # warning only!
+  # No truncation check for strip transects
+  if(!is.null(ddfobject) & !is.null(obsdata[[distance.name]])){
+    if(any(obsdata[,distance.name]>ddfobject$meta.data$width)){
+      stop("Some observations are outside of detection function truncation!")
+    }
   }
 
   # Estimating group abundance/density
@@ -36,16 +38,16 @@ make.data <- function(response, ddfobject, segdata, obsdata, group,
 
   # if the ps are all the same (count model) then just grab the 1 unique
   # value
-  if(response %in% c("N","abundance")){
+  if(response %in% c("N","abundance","count","n")){
     fitted.p <- unique(fitted.p)
   }
 
   ## Aggregate response values of the sightings over segments
-  if(response %in% c("D","density")){
+  if(response %in% c("D","density","Dhat","density.est")){
     responsedata <- aggregate(obsdata[,cluster.name]/(fitted.p*availability),
                                 list(obsdata[,segnum.name]), sum)
     off.set <- "none"
-  }else if(response %in% c("N","abundance")){
+  }else if(response %in% c("N","abundance","count","n")){
     responsedata <- aggregate(obsdata[,cluster.name]/availability,
                               list(obsdata[,segnum.name]), sum)
     off.set <- "eff.area"
@@ -61,6 +63,14 @@ make.data <- function(response, ddfobject, segdata, obsdata, group,
     off.set <- "none"
   }
 
+  ## warn if any observations were not allocated
+  responsecheck <- aggregate(obsdata[,cluster.name],
+                             list(obsdata[,segnum.name]), sum)
+  if(sum(obsdata[,cluster.name]) != sum(responsecheck[,2])){
+    message(paste0("Some observations were not allocated to segments!\n",
+                   "Check that Sample.Labels match"))
+  }
+
   # name the response data columns
   names(responsedata)<-c(segnum.name,response)
 
@@ -87,6 +97,9 @@ make.data <- function(response, ddfobject, segdata, obsdata, group,
       dat[,response] <- dat[,response]/(segment.area*convert.units)
     }
 
+    # set the segment area in the data
+    dat$segment.area <- segment.area*convert.units
+
   }else{
     # pull this from the detection function
     if (!is.null(ddfobject)){
@@ -122,10 +135,11 @@ make.data <- function(response, ddfobject, segdata, obsdata, group,
       dat[,response] <- dat[,response]/(2*dat[,seglength.name]*
                                         width*convert.units)
     }
+
+    # set the segment area in the data
+    dat$segment.area <- 2*dat[,seglength.name]*width*convert.units
   }
 
-
-
   # multiply up by conversion factor
   dat$off.set <- dat$off.set*convert.units
 
